package eu.mhsl.craftattack.spawn.api;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import org.bukkit.configuration.ConfigurationSection;
import spark.Request;
import spark.Spark;

import java.util.function.Function;
import java.util.function.Supplier;

public class HttpServer {
    private final ConfigurationSection apiConf = Main.instance().getConfig().getConfigurationSection("api");
    protected final Gson gson = new Gson();

    public static Object nothing = null;

    public HttpServer() {
        Spark.port(8080);

        Spark.get("/ping", (request, response) -> System.currentTimeMillis());

        Main.instance().getAppliances().forEach(appliance -> appliance.httpApi(new ApiBuilder(appliance)));
    }

    public void stop() {
        Spark.stop();
    }

    public record Response(Status status, Object error, Object response) {
        public enum Status {
            FAILURE,
            SUCCESS
        }
    }

    public class ApiBuilder {
        @FunctionalInterface
        public interface RequestProvider<TParsed, TOriginal, TResponse> {
            TResponse apply(TParsed parsed, TOriginal original);
        }

        private final String applianceName;

        private ApiBuilder(Appliance appliance) {
            this.applianceName = appliance.getClass().getSimpleName().toLowerCase();
        }

        public void get(String path, Function<Request, Object> onCall) {
            Spark.get(this.buildRoute(path), (req, resp) -> this.process(() -> onCall.apply(req)));
        }

        public void rawPost(String path, Function<Request, Object> onCall) {
            Spark.post(this.buildRoute(path), (req, resp) -> this.process(() -> onCall.apply(req)));
        }

        public <TRequest> void post(String path, Class<TRequest> clazz, RequestProvider<TRequest, Request, Object> onCall) {
            Spark.post(this.buildRoute(path), (req, resp) -> {
                Main.instance().getLogger().info(req.body());
                TRequest parsed = HttpServer.this.gson.fromJson(req.body(), clazz);
                return this.process(() -> onCall.apply(parsed, req));
            });
        }

        public String buildRoute(String path) {
            return String.format("/api/%s/%s", this.applianceName, path);
        }

        private String process(Supplier<Object> exec) {
            HttpServer.Response response;
            try {
                response = new Response(Response.Status.SUCCESS, null, exec.get());
            } catch(Exception e) {
                response = new Response(Response.Status.FAILURE, e, null);
            }
            return HttpServer.this.gson.toJson(response);
        }
    }
}
