package eu.mhsl.craftattack.spawn.appliances.debug.command;

import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.debug.Debug;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Stream;

public class UserInfoCommand extends ApplianceCommand<Debug> {
    public UserInfoCommand() {
        super("userInfo");
    }

    @Override
    protected void execute(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        if(args.length != 1) {
            sender.sendMessage(Component.text("Bitte gib einen Nutzernamen an.", NamedTextColor.RED));
            return;
        }

        OfflinePlayer player = Bukkit.getOfflinePlayer(args[0]);

        sender.sendMessage(
            Component.text()
                .appendNewline()
                .append(Component.text("Informationen zu: ", NamedTextColor.GOLD))
                .append(
                    Component
                        .text(Objects.requireNonNull(player.getName()), NamedTextColor.YELLOW)
                        .clickEvent(ClickEvent.copyToClipboard(Objects.requireNonNull(player.getName())))
                )
                .appendNewline()
                .append(
                    Component
                        .text("UUID: " + player.getUniqueId(), NamedTextColor.GRAY)
                        .clickEvent(ClickEvent.copyToClipboard(player.getUniqueId().toString()))
                )
                .appendNewline()
                .append(
                    Component
                        .text("Erster Besuch: " + formatUnixTimestamp(player.getFirstPlayed()), NamedTextColor.GRAY)
                        .clickEvent(ClickEvent.copyToClipboard(String.valueOf(player.getFirstPlayed())))
                )
                .appendNewline()
                .append(
                    Component
                        .text("Letzter Besuch: " + formatUnixTimestamp(player.getLastSeen()), NamedTextColor.GRAY)
                        .clickEvent(ClickEvent.copyToClipboard(String.valueOf(player.getLastSeen())))
                )
                .appendNewline()
        );
    }

    @Override
    public @Nullable List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        if(args.length < 2) {
            return Stream.concat(
                Bukkit.getOnlinePlayers().stream().map(Player::getName),
                Arrays.stream(Bukkit.getOfflinePlayers()).map(OfflinePlayer::getName)
            ).toList();
        }

        return new ArrayList<>();
    }

    private String formatUnixTimestamp(long timestamp) {
        DateFormat format = new SimpleDateFormat("E dd.MM.yyyy H:m:s");
        return format.format(new Date(timestamp));
    }
}
