package eu.mhsl.craftattack.spawn.appliances.event;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.api.HttpServer;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.event.command.*;
import eu.mhsl.craftattack.spawn.appliances.event.listener.ApplyPendingRewardsListener;
import eu.mhsl.craftattack.spawn.util.IteratorUtil;
import eu.mhsl.craftattack.spawn.util.entity.DisplayVillager;
import eu.mhsl.craftattack.spawn.util.listener.DismissInventoryOpenFromHolder;
import eu.mhsl.craftattack.spawn.util.listener.PlayerInteractAtEntityEventListener;
import eu.mhsl.craftattack.spawn.util.server.PluginMessage;
import eu.mhsl.craftattack.spawn.util.text.ComponentUtil;
import eu.mhsl.craftattack.spawn.util.text.Countdown;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.entity.Villager;
import org.bukkit.event.Listener;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.*;

public class Event extends Appliance {
    enum AdvertisementStatus {
        BEFORE,
        ADVERTISED,
        DONE
    }

    Countdown advertiseCountdown = new Countdown(
        120,
        announcementData -> Component.text()
            .append(ComponentUtil.createRainbowText("Event", 30))
            .append(Component.text(" Start in ", NamedTextColor.GOLD))
            .append(Component.text(announcementData.count(), NamedTextColor.AQUA))
            .append(Component.text(" " + announcementData.unit() + "!", NamedTextColor.GOLD))
            .build(),
        component -> IteratorUtil.onlinePlayers(player -> player.sendMessage(component)),
        () -> this.advertiseStatus = AdvertisementStatus.DONE
    );
    public DisplayVillager.ConfigBound villager;
    private boolean isOpen = false;
    private AdvertisementStatus advertiseStatus = AdvertisementStatus.BEFORE;
    private UUID roomId;
    private final HttpClient eventServerClient = HttpClient.newHttpClient();
    private final List<Reward> pendingRewards = new ArrayList<>();

    record RewardConfiguration(String memorialMaterial, String memorialTitle, String memorialLore, List<UUID> memorials,
                               String material, Map<UUID, Integer> rewards) {
    }

    record Reward(UUID playerUuid, ItemStack itemStack) {
    }

    public Event() {
        super("event");
    }

    @Override
    public void onEnable() {
        this.villager = new DisplayVillager.ConfigBound(
            localConfig(),
            villager -> {
                villager.customName(Component.text("Events", NamedTextColor.GOLD));
                villager.setProfession(Villager.Profession.LIBRARIAN);
                villager.setVillagerType(Villager.Type.SNOW);
            }
        );
        this.isOpen = localConfig().getBoolean("enabled", false);
        if(this.isOpen) this.roomId = UUID.fromString(localConfig().getString("roomId", ""));
    }

    public void openEvent() throws URISyntaxException, IOException, InterruptedException {
        if(isOpen) throw new ApplianceCommand.Error("Es läuft derzeit bereits ein Event!");
        HttpRequest createRoomRequest = HttpRequest.newBuilder()
            .uri(new URI(localConfig().getString("api") + "/room"))
            .POST(HttpRequest.BodyPublishers.noBody())
            .build();

        HttpResponse<String> rawResponse = eventServerClient.send(createRoomRequest, HttpResponse.BodyHandlers.ofString());
        if(rawResponse.statusCode() != 200)
            throw new ApplianceCommand.Error("Event-Server meldet Fehler: " + rawResponse.statusCode());

        record Response(UUID uuid) {
        }
        Response response = new Gson().fromJson(rawResponse.body(), Response.class);

        isOpen = true;
        roomId = response.uuid;
    }

    public void joinEvent(Player p) {
        if(!isOpen) {
            p.sendMessage(Component.text("Zurzeit ist kein Event geöffnet.", NamedTextColor.RED));
            return;
        }

        if(!p.hasPermission("admin") && advertiseStatus == AdvertisementStatus.BEFORE) {
            p.sendMessage(Component.text("Die Event befinden sich noch in der Vorbereitung.", NamedTextColor.RED));
            return;
        }

        if(!p.hasPermission("admin") && advertiseStatus == AdvertisementStatus.DONE) {
            p.sendMessage(Component.text("Die Events laufen bereits. Ein nachträgliches Beitreten ist nicht möglich.", NamedTextColor.RED));
            return;
        }

        try {
            Main.instance().getLogger().info("Verbinde mit eventserver: " + p.getName());
            p.sendMessage(Component.text("Authentifiziere...", NamedTextColor.GREEN));
            record Request(UUID player, UUID room) {
            }
            Request request = new Request(p.getUniqueId(), this.roomId);
            HttpRequest queueRoomRequest = HttpRequest.newBuilder()
                .uri(new URI(localConfig().getString("api") + "/queueRoom"))
                .POST(HttpRequest.BodyPublishers.ofString(new Gson().toJson(request)))
                .build();

            record Response(String error) {
            }
            HttpResponse<String> rawResponse = eventServerClient.send(queueRoomRequest, HttpResponse.BodyHandlers.ofString());
            Main.instance().getLogger().info("Response: " + rawResponse.body());
            Response response = new Gson().fromJson(rawResponse.body(), Response.class);

            if(rawResponse.statusCode() != 200 || response.error != null) {
                p.sendMessage(Component.text("Fehler beim Betreten: " + response.error, NamedTextColor.RED));
                return;
            }

            p.sendMessage(Component.text("Betrete...", NamedTextColor.GREEN));
            PluginMessage.connect(p, localConfig().getString("connect-server-name"));

        } catch(URISyntaxException | IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public void endEvent() {
        if(!isOpen) throw new ApplianceCommand.Error("Es läuft derzeit kein Event!");
        isOpen = false;
    }

    private void rewardPlayers(RewardConfiguration rewardConfiguration) {
        rewardConfiguration.rewards.forEach((uuid, amount) -> {
            Reward reward = new Reward(uuid, new ItemStack(Objects.requireNonNull(Material.matchMaterial(rewardConfiguration.material)), amount));

            if(Bukkit.getPlayer(uuid) == null) {
                pendingRewards.add(reward);
                return;
            }
            giveReward(reward);
        });

        rewardConfiguration.memorials.forEach(uuid -> {
            ItemStack memorialItem = new ItemStack(Objects.requireNonNull(Material.matchMaterial(rewardConfiguration.memorialMaterial)));
            ItemMeta meta = memorialItem.getItemMeta();
            meta.displayName(Component.text(rewardConfiguration.memorialTitle, NamedTextColor.GOLD));
            meta.lore(List.of(Component.text(rewardConfiguration.memorialLore, NamedTextColor.AQUA)));
            memorialItem.setItemMeta(meta);
            Reward memorial = new Reward(uuid, memorialItem);

            if(Bukkit.getPlayer(uuid) == null) {
                pendingRewards.add(memorial);
                return;
            }
            giveReward(memorial);
        });
    }

    private void giveReward(Reward reward) {
        Player player = Bukkit.getPlayer(reward.playerUuid);
        if(player == null) throw new RuntimeException("Cannot reward offline playerUuid!");

        Map<Integer, ItemStack> remaining = player.getInventory().addItem(reward.itemStack);
        Bukkit.getScheduler().runTask(
            Main.instance(),
            () -> remaining.values().forEach(remainingStack -> player.getWorld().dropItem(player.getLocation(), remainingStack))
        );
    }

    public void applyPendingRewards(Player player) {
        if(this.pendingRewards.isEmpty()) return;
        List<Reward> givenRewards = this.pendingRewards.stream().filter(reward -> reward.playerUuid.equals(player.getUniqueId())).toList();
        this.pendingRewards.removeAll(givenRewards);
        givenRewards.forEach(this::giveReward);
    }

    public void advertise() {
        advertiseCountdown.cancelIfRunning();
        this.advertiseStatus = AdvertisementStatus.ADVERTISED;
        IteratorUtil.onlinePlayers(player -> {
            player.sendMessage(
                Component.text()
                    .append(Component.text("-".repeat(10), NamedTextColor.GRAY)).appendNewline()
                    .append(Component.text("Ein Event wurde gestartet!", NamedTextColor.GOLD)).appendNewline()
                    .append(Component.text("Nutze "))
                    .append(Component.text("/event", NamedTextColor.AQUA))
                    .append(Component.text(", um dem Event beizutreten!")).appendNewline()
                    .append(Component.text("-".repeat(10), NamedTextColor.GRAY)).appendNewline()
            );
        });
        advertiseCountdown.start();
    }

    @Override
    public void httpApi(HttpServer.ApiBuilder apiBuilder) {
        apiBuilder.post("reward", RewardConfiguration.class, (rewardConfiguration, request) -> {
            this.rewardPlayers(rewardConfiguration);
            return HttpServer.nothing;
        });
    }

    @Override
    @NotNull
    protected List<ApplianceCommand<?>> commands() {
        return List.of(
            new EventCommand(),
            new MoveEventVillagerCommand(),
            new EventOpenSessionCommand(),
            new EventEndSessionCommand(),
            new EventAdvertiseCommand()
        );
    }

    @Override
    @NotNull
    protected List<Listener> eventHandlers() {
        return List.of(
            new ApplyPendingRewardsListener(),
            new PlayerInteractAtEntityEventListener(this.villager.getUniqueId(), playerInteractAtEntityEvent -> joinEvent(playerInteractAtEntityEvent.getPlayer())),
            new DismissInventoryOpenFromHolder(this.villager.getUniqueId())
        );
    }
}
