package eu.mhsl.craftattack.spawn.appliances.report;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.UUID;

public class Report extends Appliance {
    public static Component helpText() {
        return Component.text()
            .appendNewline()
            .append(Component.text(" Um einen Spieler zu melden, verwende ", NamedTextColor.GRAY)).appendNewline()
            .append(Component.text("/report", NamedTextColor.GOLD)).appendNewline()
            .append(Component.text("oder", NamedTextColor.GRAY)).appendNewline()
            .append(Component.text("/report <spieler> [grund]", NamedTextColor.GOLD)).appendNewline()
            .build();
    }

    private final URI apiEndpoint;

    public Report() {
        super("report");
        this.apiEndpoint = URI.create(Objects.requireNonNull(localConfig().getString("api")));
    }

    private record Request(@NotNull UUID reporter, @Nullable UUID reported, String reason) {
    }

    private record Response(@NotNull String url) {
    }

    public void reportToUnknown(@NotNull Player issuer) {
        Request request = new Request(issuer.getUniqueId(), null, "");
        this.issueReport(issuer, request);
    }

    public void reportToKnown(@NotNull Player issuer, @NotNull String targetUsername, @Nullable String reason) {
        OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(targetUsername);
        if(issuer.getUniqueId().equals(offlinePlayer.getUniqueId())) {
            issuer.sendMessage(
                Component.text("Du kannst dich nicht selbst reporten.").color(NamedTextColor.RED)
            );
            return;
        }

        Request request = new Request(issuer.getUniqueId(), offlinePlayer.getUniqueId(), Optional.ofNullable(reason).orElse(""));
        this.issueReport(issuer, request);
    }

    private void issueReport(Player issuer, Request reportRequest) {
        try(HttpClient client = HttpClient.newHttpClient()) {
            HttpRequest httpRequest = HttpRequest.newBuilder()
                .uri(this.apiEndpoint)
                .header("Content-Type", "application/json")
                .POST(HttpRequest.BodyPublishers.ofString(new Gson().toJson(reportRequest)))
                .build();

            HttpResponse<String> httpResponse = client.send(httpRequest, HttpResponse.BodyHandlers.ofString());
            this.printResultMessage(issuer, httpResponse);
        } catch(IOException | InterruptedException e) {
            issuer.sendMessage(
                Component.text("Internal server description: " + e.getMessage()).color(NamedTextColor.RED)
            );
            throw new RuntimeException(e);
        }
    }

    private void printResultMessage(Player issuer, HttpResponse<String> httpResponse) {
        switch(httpResponse.statusCode()) {
            case 201:
                Response response = new Gson().fromJson(httpResponse.body(), Response.class);
                issuer.sendMessage(
                    Component.text()
                        .append(Component.text("\\/".repeat(20), NamedTextColor.DARK_GRAY))
                        .appendNewline()
                        .appendNewline()
                        .append(Component.text("!!! Report hier fertigstellen !!!", NamedTextColor.GOLD))
                        .appendNewline()
                        .appendNewline()
                        .append(
                            Component
                                .text(" > Hier klicken < ", NamedTextColor.GREEN)
                                .hoverEvent(HoverEvent.showText(Component.text(response.url).color(NamedTextColor.GREEN)))
                                .clickEvent(ClickEvent.clickEvent(ClickEvent.Action.OPEN_URL, response.url))
                        )
                        .appendNewline()
                        .appendNewline()
                        .append(
                            Component
                                .text(response.url, NamedTextColor.GRAY)
                                .clickEvent(ClickEvent.clickEvent(ClickEvent.Action.OPEN_URL, response.url))
                        )
                        .appendNewline()
                        .appendNewline()
                        .append(Component.text("/\\".repeat(20), NamedTextColor.DARK_GRAY))
                );
                break;

            case 400:
                issuer.sendMessage(
                    Component.text()
                        .append(Component.text("Der angegebene Nutzer ist in unserem System nicht bekannt.", NamedTextColor.RED))
                        .appendNewline()
                        .append(Component.text("Bist du sicher, dass du den Namen richtig geschrieben hast?", NamedTextColor.RED))
                        .appendNewline()
                        .append(Component.text("Du kannst dich alternativ jederzeit bei einem Admin melden.", NamedTextColor.GRAY))
                );
                break;

            case 401:
            default:
                Main.logger().warning("Failed to request Report: " + httpResponse.statusCode());
                issuer.sendMessage(
                    Component.text()
                        .append(Component.text("Interner Serverfehler beim anlegen des Reports.", NamedTextColor.RED))
                        .appendNewline()
                        .append(Component.text("Bitte melde dich bei einem Admin!", NamedTextColor.RED))
                );
                break;
        }
    }

    @Override
    @NotNull
    protected List<ApplianceCommand<?>> commands() {
        return List.of(new ReportCommand());
    }
}
