package eu.mhsl.craftattack.spawn.appliances.settings.datatypes;

import eu.mhsl.craftattack.spawn.appliances.settings.Settings;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

public abstract class BoolSetting extends Setting<Boolean> {
    public BoolSetting(Settings.Key key) {
        super(key);
    }

    @Override
    public void fromStorage(PersistentDataContainer container) {
        this.state = container.has(getNamespacedKey())
            ? Objects.requireNonNull(container.get(getNamespacedKey(), PersistentDataType.BOOLEAN))
            : defaultValue();
    }

    @Override
    protected void toStorage(PersistentDataContainer container, Boolean value) {
        container.set(getNamespacedKey(), PersistentDataType.BOOLEAN, value);
    }

    @Override
    public ItemMeta buildMeta(ItemMeta meta) {
        meta.displayName(Component.text(title(), NamedTextColor.WHITE));
        List<Component> lore = new ArrayList<>(List.of(
            Component.empty()
                .append(Component.text("Status: ", NamedTextColor.DARK_GRAY))
                .append(Component.text(
                    this.state ? "Aktiviert" : "Deaktiviert",
                    this.state ? NamedTextColor.GREEN : NamedTextColor.RED)
                ),
            Component.empty()
        ));
        lore.addAll(buildDescription(description()));
        meta.lore(lore);
        return meta;
    }

    @Override
    protected void change(ClickType clickType) {
        this.state = !this.state;
    }

    @Override
    public Class<?> dataType() {
        return Boolean.class;
    }

    @Override
    public Boolean state() {
        return state;
    }
}
