package eu.mhsl.craftattack.spawn.appliances.worldmuseum;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.util.entity.DisplayVillager;
import eu.mhsl.craftattack.spawn.util.listener.DismissInventoryOpenFromHolder;
import eu.mhsl.craftattack.spawn.util.listener.PlayerInteractAtEntityEventListener;
import eu.mhsl.craftattack.spawn.util.server.Floodgate;
import eu.mhsl.craftattack.spawn.util.server.PluginMessage;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.entity.Villager;
import org.bukkit.event.Listener;
import org.geysermc.cumulus.form.SimpleForm;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class WorldMuseum extends Appliance {
    public DisplayVillager.ConfigBound villager;

    public WorldMuseum() {
        super("worldMuseum");
    }

    @Override
    public void onEnable() {
        this.villager = new DisplayVillager.ConfigBound(
            localConfig(),
            villager -> {
                villager.customName(Component.text("Museum der Welten").color(NamedTextColor.GOLD));
                villager.setProfession(Villager.Profession.CARTOGRAPHER);
                villager.setVillagerType(Villager.Type.SNOW);
            }
        );
    }

    public void updateVillagerPosition(Location location) {
        this.villager.updateLocation(location);
    }

    public void handleVillagerInteraction(Player player) {
        if(Floodgate.isBedrock(player)) {
            Floodgate.runBedrockOnly(player, floodgatePlayer -> {
                floodgatePlayer.sendForm(
                    SimpleForm.builder()
                        .title("Nicht unterstützt")
                        .content("Bedrock-Spieler werden derzeit für das Weltenmuseum aus Kompatiblitätsgründen nicht zugelassen! Tut uns Leid.")
                        .button("Ok")
                        .build()
                );
            });
            return;
        }

        Main.logger().info("Sending" + player.getName() + " to WorldMuseum");
        PluginMessage.connect(player, localConfig().getString("connect-server-name"));
    }

    @Override
    @NotNull
    protected List<ApplianceCommand<?>> commands() {
        return List.of(new MoveWorldMuseumVillagerCommand());
    }

    @Override
    @NotNull
    protected List<Listener> eventHandlers() {
        return List.of(
            new PlayerInteractAtEntityEventListener(this.villager.getUniqueId(), playerInteractAtEntityEvent -> handleVillagerInteraction(playerInteractAtEntityEvent.getPlayer())),
            new DismissInventoryOpenFromHolder(this.villager.getUniqueId())
        );
    }
}
