package eu.mhsl.craftattack.spawn.util.statistics;

import eu.mhsl.craftattack.spawn.Main;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.scheduler.BukkitTask;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.Duration;

public class NetworkMonitor {
    private long previousRxBytes = 0;
    private long previousTxBytes = 0;

    private long previousRxPackets = 0;
    private long previousTxPackets = 0;

    private final String iFace;

    private long rxBytesLastDuration = 0;
    private long txBytesLastDuration = 0;
    private long rxPacketsLastDuration = 0;
    private long txPacketsLastDuration = 0;

    private final BukkitTask updateTask;

    public NetworkMonitor(String iFace, Duration sampleDuration) {
        this.iFace = iFace;

        this.updateTask = Bukkit.getScheduler().runTaskTimerAsynchronously(
            Main.instance(),
            this::update,
            0,
            sampleDuration.getSeconds() * Ticks.TICKS_PER_SECOND
        );
    }

    public record Traffic(long rxBytes, long txBytes) {
    }

    public record Packets(long rxCount, long txCount) {
    }

    public Traffic getTraffic() {
        return new Traffic(rxBytesLastDuration, txBytesLastDuration);
    }

    public Packets getPackets() {
        return new Packets(rxPacketsLastDuration, txPacketsLastDuration);
    }

    public void stop() {
        this.updateTask.cancel();
    }

    private void update() {
        long rxBytes = getNetworkStatistic("rx_bytes");
        long txBytes = getNetworkStatistic("tx_bytes");
        long rxPackets = getNetworkStatistic("rx_packets");
        long txPackets = getNetworkStatistic("tx_packets");

        this.rxBytesLastDuration = rxBytes - previousRxBytes;
        this.txBytesLastDuration = txBytes - previousTxBytes;
        this.rxPacketsLastDuration = rxPackets - previousRxPackets;
        this.txPacketsLastDuration = txPackets - previousTxPackets;

        this.previousRxBytes = rxBytes;
        this.previousTxBytes = txBytes;
        this.previousRxPackets = rxPackets;
        this.previousTxPackets = txPackets;
    }

    private long getNetworkStatistic(String statistic) {
        try {
            String path = String.format("/sys/class/net/%s/statistics/%s", this.iFace, statistic);
            String content = new String(Files.readAllBytes(Paths.get(path)));
            return Long.parseLong(content.trim());
        } catch(IOException e) {
            throw new RuntimeException("Failed recieving Network statistic", e);
        }
    }
}
