package eu.mhsl.craftattack.spawn.util.text;

import eu.mhsl.craftattack.spawn.util.statistics.NetworkMonitor;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import java.awt.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class ComponentUtil {
    public static TextComponent appendWithNewline(Component a, Component b) {
        return Component.text().append(a.appendNewline().append(b)).build();
    }

    public static Stream<String> lineBreak(String text, int charactersPerLine) {
        List<String> lines = new ArrayList<>();
        String[] words = text.split(" ");
        StringBuilder line = new StringBuilder();

        for(String word : words) {
            if(line.length() + word.length() + 1 > charactersPerLine) {
                lines.add(line.toString().trim());
                line = new StringBuilder();
            }
            line.append(word).append(" ");
        }

        if(!line.isEmpty()) {
            lines.add(line.toString().trim());
        }

        return lines.stream();
    }

    public static String lineBreakNL(String text, int charactersPerLine) {
        Stream<String> lines = lineBreak(text, charactersPerLine);
        return lines.collect(Collectors.joining("\n"));
    }

    public static Component getFormattedTPS() {
        double[] tpsValues = Bukkit.getTPS();

        double min1 = Math.min(1.0, Math.max(0.0, tpsValues[0] / 20.0));
        double min2 = Math.min(1.0, Math.max(0.0, tpsValues[1] / 20.0));
        double min3 = Math.min(1.0, Math.max(0.0, tpsValues[2] / 20.0));

        int red1 = (int) (255 * (1.0 - min1));
        int green1 = (int) (255 * min1);
        int red2 = (int) (255 * (1.0 - min2));
        int green2 = (int) (255 * min2);
        int red3 = (int) (255 * (1.0 - min3));
        int green3 = (int) (255 * min3);

        TextColor tpsColor1 = TextColor.color(red1, green1, 0);
        TextColor tpsColor2 = TextColor.color(red2, green2, 0);
        TextColor tpsColor3 = TextColor.color(red3, green3, 0);

        return Component.text()
            .append(Component.text("TPS 1, 5, 15m: ", NamedTextColor.GRAY))
            .append(Component.text(String.format("%.2f", tpsValues[0]), tpsColor1))
            .append(Component.text(", "))
            .append(Component.text(String.format("%.2f", tpsValues[1]), tpsColor2))
            .append(Component.text(", "))
            .append(Component.text(String.format("%.2f", tpsValues[2]), tpsColor3))
            .build();
    }

    public static Component getFormattedTickTimes(boolean detailed) {
        long[] times = Bukkit.getServer().getTickTimes();
        float mspt = ((float) Arrays.stream(times).sum() / times.length) * 1.0E-6f;
        float roundedMspt = Math.round(mspt * 100f) / 100f;
        int loadPercentage = (int) (Math.min(100, (mspt / 50.0) * 100));
        float roundedTPS = Math.round(Bukkit.getTPS()[0] * 100f) / 100f;

        TextColor msptColor = ColorUtil.mapGreenToRed(roundedMspt, 0, 50, true);
        TextColor percentageColor = ColorUtil.mapGreenToRed(loadPercentage, 80, 100, true);
        TextColor tpsColor = ColorUtil.mapGreenToRed(roundedTPS, 15, 20, false);

        ComponentBuilder<TextComponent, TextComponent.Builder> tickTimes = Component.text()
            .append(Component.text("Serverlast: ", NamedTextColor.GRAY))
            .append(Component.text(loadPercentage + "% ", percentageColor))
            .appendNewline();

        if(detailed) {
            tickTimes
                .append(Component.text(roundedMspt + "mspt", msptColor))
                .append(Component.text(" | ", NamedTextColor.GRAY));
        }

        return tickTimes
            .append(Component.text(roundedTPS + "tps", tpsColor))
            .build();
    }


    public static Component getFormattedPing(Player player) {
        int playerPing = player.getPing();
        int averagePing = Bukkit.getOnlinePlayers().stream()
            .map(Player::getPing).reduce(Integer::sum)
            .orElse(0) / Bukkit.getOnlinePlayers().size();

        return Component.text()
            .append(Component.text("Dein Ping: ", NamedTextColor.GRAY))
            .append(Component.text(playerPing + "ms", ColorUtil.mapGreenToRed(playerPing, 50, 200, true)))
            .append(Component.text(" | ", NamedTextColor.GRAY))
            .append(Component.text("Durschnitt: ", NamedTextColor.GRAY))
            .append(Component.text(averagePing + "ms", ColorUtil.mapGreenToRed(averagePing, 50, 200, true)))
            .build();
    }

    public static Component createRainbowText(String text, int step) {
        Component builder = Component.empty();
        int hue = 0;
        for(char c : text.toCharArray()) {
            TextColor color = TextColor.color(Color.getHSBColor((float) hue / 360, 1, 1).getRGB());
            builder = builder.append(Component.text(c).color(color));
            hue += step;
        }

        return builder;
    }

    public static Component getFormattedNetworkStats(NetworkMonitor.Traffic traffic, NetworkMonitor.Packets packets) {
        return Component.text()
            .append(Component.text(
                DataSizeConverter.convertBytesToHumanReadable(traffic.rxBytes()) + " ↓ " + NumberAbbreviation.abbreviateNumber(packets.rxCount()) + "pps",
                NamedTextColor.GREEN
            ))
            .append(Component.text(" | ", NamedTextColor.GRAY))
            .append(Component.text(
                DataSizeConverter.convertBytesToHumanReadable(traffic.txBytes()) + " ↑ " + NumberAbbreviation.abbreviateNumber(packets.rxCount()) + "pps",
                NamedTextColor.RED
            ))
            .build();
    }
}
