package eu.mhsl.craftattack.spawn.util.text;

import eu.mhsl.craftattack.spawn.Main;
import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;

public class Countdown {
    private final int countdownFrom;
    private boolean running;
    private int taskId;
    private int current;
    private final Consumer<Component> announcementConsumer;
    private final Function<AnnouncementData, Component> announcementBuilder;
    private Function<Integer, AnnouncementData> defaultAnnouncements;
    private final List<CustomAnnouncements> customAnnouncements = new ArrayList<>();
    private final Runnable onDone;


    public record AnnouncementData(int count, String unit) {
    }

    public record CustomAnnouncements(Function<Integer, Boolean> test, Consumer<Integer> task) {
    }

    public Countdown(int countdownFrom, Function<AnnouncementData, Component> announcementBuilder, Consumer<Component> announcementConsumer, Runnable onDone) {
        this.countdownFrom = countdownFrom;
        this.current = countdownFrom;
        this.announcementBuilder = announcementBuilder;
        this.announcementConsumer = announcementConsumer;
        this.onDone = onDone;

        this.defaultAnnouncements = count -> {
            if(current > 60 && current % 60 == 0) {
                return new AnnouncementData(current / 60, "Minuten");
            }

            if(current <= 60 && (current <= 10 || current % 10 == 0)) {
                return new AnnouncementData(current, "Sekunden");
            }

            return null;
        };
    }

    public void addCustomAnnouncement(CustomAnnouncements announcement) {
        this.customAnnouncements.add(announcement);
    }

    public void setDefaultAnnouncements(Function<Integer, AnnouncementData> defaultAnnouncement) {
        this.defaultAnnouncements = defaultAnnouncement;
    }

    public void start() {
        if(this.running) throw new IllegalStateException("Countdown already running!");
        this.running = true;
        this.taskId = Bukkit.getScheduler().scheduleSyncRepeatingTask(Main.instance(), this::tick, 20, 20);
    }

    public void cancel() {
        if(!this.running) throw new IllegalStateException("Countdown not running!");
        this.running = false;
        Bukkit.getScheduler().cancelTask(this.taskId);
        this.taskId = 0;
        this.current = countdownFrom;
    }

    public void cancelIfRunning() {
        if(this.running) cancel();
    }

    private void tick() {
        AnnouncementData defaultAnnouncementData = this.defaultAnnouncements.apply(current);
        if(defaultAnnouncementData != null) {
            this.announcementConsumer.accept(this.announcementBuilder.apply(defaultAnnouncementData));
        }

        customAnnouncements
            .stream()
            .filter(a -> a.test.apply(current))
            .forEach(a -> a.task.accept(current));

        this.current--;

        if(isDone()) {
            this.onDone.run();
            this.cancel();
        }
    }

    public boolean isDone() {
        return current <= 0;
    }

    public boolean isRunning() {
        return this.running;
    }
}
